@@ -24,13 +24,14 @@ import inspect
 import textwrap
 import warnings
 import operator
+import pathlib
 from collections import namedtuple
 from contextlib import contextmanager
 
 import numpy as np
 from numpy.lib.stride_tricks import as_strided
 import astropy.units as u
-from astropy.utils import lazyproperty
+from astropy.utils import lazyproperty, isiterable
 
 from ..helpers import sequentialfile as sf
 from .offsets import RawOffsets
@@ -830,8 +831,17 @@ class StreamReaderBase(StreamBase):
         """Value to use for invalid or missing data. Default: 0."""
         return self._fill_value
 
+    @property
+    def dtype(self):
+        # TODO: arguably, this should be inferred from an actual payload.
+        return np.dtype(np.complex64 if self.complex_data else np.float32)
+
+    def readable(self):
+        """Whether the file can be read and decoded."""
+        return self.fh_raw.readable and self.fh_raw.info.readable
+
     def seek(self, offset, whence=0):
-        """Change the stream position.
+        """Change the sample pointer position.
 
         This works like a normal filehandle seek, but the offset is in samples
         (or a relative or absolute time).
@@ -840,7 +850,8 @@ class StreamReaderBase(StreamBase):
         ----------
         offset : int, `~astropy.units.Quantity`, or `~astropy.time.Time`
             Offset to move to.  Can be an (integer) number of samples,
-            an offset in time units, or an absolute time.
+            an offset in time units, or an absolute time.  For the latter
+            two, the pointer will be moved to the nearest integer sample.
         whence : {0, 1, 2, 'start', 'current', or 'end'}, optional
             Like regular seek, the offset is taken to be from the start if
             ``whence=0`` (default), from the current position if 1,
@@ -865,48 +876,39 @@ class StreamReaderBase(StreamBase):
         elif whence == 1 or whence == 'current':
             self.offset += offset
         elif whence == 2 or whence == 'end':
-            self.offset = self._nsample + offset
+            self.offset = self.shape[0] + offset
         else:
             raise ValueError("invalid 'whence'; should be 0 or 'start', 1 or "
                              "'current', or 2 or 'end'.")
 
         return self.offset
 
-    @property
-    def dtype(self):
-        # TODO: arguably, this should be inferred from an actual payload.
-        return np.dtype(np.complex64 if self.complex_data else np.float32)
-
-    def readable(self):
-        """Whether the file can be read and decoded."""
-        return self.fh_raw.readable and self.fh_raw.info.readable
-
     def read(self, count=None, out=None):
-        """Read a number of complete (or subset) samples.
-
-        The range retrieved can span multiple frames.
+        """Read a number of complete samples.
 
         Parameters
         ----------
         count : int or None, optional
-            Number of complete/subset samples to read.  If `None` (default) or
-            negative, the whole file is read.  Ignored if ``out`` is given.
+            Number of complete samples to read. If `None` (default) or
+            negative, the number of samples left. Ignored if ``out`` is given.
         out : None or array, optional
-            Array to store the data in. If given, ``count`` will be inferred
-            from the first dimension; the other dimension should equal
+            Array to store the samples in. If given, ``count`` will be inferred
+            from the first dimension; the remaining dimensions should equal
             `sample_shape`.
 
         Returns
         -------
         out : `~numpy.ndarray` of float or complex
-            The first dimension is sample-time, and the remainder given by
-            `sample_shape`.
+            The first dimension is sample-time, and the remaining ones are
+            as given by `sample_shape`.
         """
+        if self.closed:
+            raise ValueError("I/O operation on closed stream.")
+
+        samples_left = self.shape[0] - self.offset
         if out is None:
             if count is None or count < 0:
-                count = self._nsample - self.offset
-                if count < 0:
-                    raise EOFError("cannot read from beyond end of file.")
+                count = max(0, samples_left)
 
             out = np.empty((count,) + self.sample_shape, dtype=self.dtype)
         else:
@@ -914,6 +916,9 @@ class StreamReaderBase(StreamBase):
                 "'out' must have trailing shape {}".format(self.sample_shape))
             count = out.shape[0]
 
+        if count > samples_left:
+            raise EOFError("cannot read from beyond end of input.")
+
         offset0 = self.offset
         sample = 0
         while sample < count:
@@ -935,8 +940,7 @@ class StreamReaderBase(StreamBase):
 
         Finds the index corresponding to the needed frame, assuming frames
         are all the same length.  If not already cached, it retrieves a
-        frame by calling ``self._new_frame(index)``.  The reader and writer
-        subclasses provide implementations for this.
+        frame by calling ``self._read_frame(index)``.
 
         Parameters
         ----------
@@ -1331,6 +1335,8 @@ class FileInfo:
             with self.open(name, mode=mode, **kwargs) as fh:
                 return fh.info
         except Exception as exc:
+            if isinstance(exc, FileNotFoundError):
+                raise
             return exc
 
     def is_ok(self, info):
@@ -1357,10 +1363,15 @@ class FileInfo:
             Information on the file.  Will evaluate as `False` if the
             file was not in the right format.
 
+        Raises
+        ------
+        FileNotFoundError
+            If the file does not exist.
+
         Notes
         -----
-        Getting information should never fail. If an `Exception` is
-        raised or returned, it is a bug in the file reader.
+        Getting information for an existing file should never fail. If an
+        `Exception` is raised or returned, it is a bug in the file reader.
         """
         info = self._get_info(name, 'rb')
         # If right format, check if arguments were missing.
@@ -1399,7 +1410,9 @@ class FileInfo:
             the opening as a stream failed.
         """
         frame_rate = file_info.frame_rate
-        used_kwargs = file_info.used_kwargs
+        used_kwargs = file_info.used_kwargs.copy()
+
+        # Somewhat inelegant special-cases for sample_rate and verify.
         if frame_rate is None:
             if 'sample_rate' in kwargs:
                 used_kwargs['sample_rate'] = kwargs['sample_rate']
@@ -1408,6 +1421,9 @@ class FileInfo:
                 # file_info.
                 return None
 
+        if 'verify' in kwargs:
+            used_kwargs['verify'] = kwargs['verify']
+
         stream_info = self._get_info(name, mode='rs', **used_kwargs)
         if self.is_ok(stream_info):
             stream_info.used_kwargs = used_kwargs
@@ -1636,19 +1652,18 @@ class FileOpener:
                          f'({self.fmt} supports {set(self.classes)}).')
 
     def _get_type(self, name):
-        if hasattr(name, 'read') or hasattr(name, 'write'):
-            return 'fh'
-
         try:
-            f0 = name[0]
-        except (TypeError, IndexError):
-            raise ValueError("name '{name}' not understood.") from None
+            pathlib.Path(name)
+        except TypeError:
+            if hasattr(name, 'read') or hasattr(name, 'write'):
+                return 'fh'
 
-        if (isinstance(name, (tuple, list, sf.FileNameSequencer))
-                or isinstance(f0, str) and len(f0) > 1):
-            return 'sequence'
+            elif isiterable(name):
+                return 'sequence'
 
-        if '{' in name and '}' in name:
+            raise ValueError("name '{name}' not understood.")
+
+        if isinstance(name, str) and '{' in name and '}' in name:
             return 'template'
         else:
             return 'name'
@@ -1759,7 +1774,8 @@ class FileOpener:
         Parameters
         ----------
         name : str or filehandle, or sequence of str
-            File name, filehandle, sequence of file names, or template.
+            File name, filehandle, sequence of file names, or template
+            (file name(s) can be `~pathlib.Path` but template has to be `str`).
         mode : {'rb', 'wb', 'rs', or 'ws'}, optional
             Whether to open for reading or writing, and as a regular binary
             file or as a stream. Default: 'rs', for reading a stream.
