@@ -3,7 +3,7 @@
 
 See http://gmrt.ncra.tifr.res.in/gmrt_hpage/sub_system/gmrt_gsb/index.htm
 """
-from .base import open  # noqa
+from .base import open, info  # noqa
 from .header import GSBHeader  # noqa
 from .payload import GSBPayload  # noqa
 from .frame import GSBFrame  # noqa
