@@ -16,8 +16,8 @@ import numpy as np
 from astropy.time import Time
 from astropy.utils import sharedmethod
 
-from ..vlbi_base.header import HeaderParser, VLBIHeaderBase
-from ..vlbi_base.utils import bcd_decode, bcd_encode, CRCStack
+from ..base.header import HeaderParser, VLBIHeaderBase
+from ..base.utils import bcd_decode, bcd_encode, fixedvalue, CRCStack
 
 __all__ = ['CRC12', 'crc12', 'stream2words', 'words2stream',
            'Mark4TrackHeader', 'Mark4Header']
@@ -291,8 +291,8 @@ class Mark4Header(Mark4TrackHeader):
 
     _track_header = Mark4TrackHeader
     _properties = (Mark4TrackHeader._properties
-                   + ('fanout', 'samples_per_frame', 'bps', 'nchan', 'nsb',
-                      'converters'))
+                   + ('fanout', 'samples_per_frame', 'bps', 'complex_data',
+                      'nchan', 'sample_shape', 'nsb', 'converters'))
     _dtypes = MARK4_DTYPES
 
     # keyed with bps, fanout; Tables 10-14 in reference documentation:
@@ -355,7 +355,7 @@ class Mark4Header(Mark4TrackHeader):
         invariants : set of str, optional
             Set of keys to header parts that are shared between all headers
             of a given type or within a given stream/file.  Default: from
-            `~baseband.vlbi_base.header.VLBIHeaderBase.invariants()`.
+            `~baseband.base.header.VLBIHeaderBase.invariants()`.
         ntrack : int, optional
             Number of tracks.  Required for getting class invariants,
             ignored for instances.
@@ -455,7 +455,7 @@ class Mark4Header(Mark4TrackHeader):
 
     def tofile(self, fh):
         stream = words2stream(self.words)
-        fh.write(stream.tostring())
+        fh.write(stream.tobytes())
 
     @classmethod
     def fromvalues(cls, ntrack, decade=None, ref_time=None, **kwargs):
@@ -627,6 +627,11 @@ class Mark4Header(Mark4TrackHeader):
             raise ValueError("Mark 4 data can only have bps=1 or 2, "
                              "not {0}".format(bps))
 
+    @fixedvalue
+    def complex_data(cls):
+        """Whether the data are complex.  Always False for Mark 4."""
+        return False
+
     @property
     def nchan(self):
         """Number of channels (``ntrack * fanout``) in the frame.
@@ -639,6 +644,15 @@ class Mark4Header(Mark4TrackHeader):
     def nchan(self, nchan):
         self.bps = self.ntrack // (self.fanout * nchan)
 
+    @property
+    def sample_shape(self):
+        """Shape of a sample in the payload (nchan,)."""
+        return (self.nchan,)
+
+    @sample_shape.setter
+    def sample_shape(self, sample_shape):
+        self.nchan, = sample_shape
+
     @property
     def nsb(self):
         """Number of side bands used.
@@ -786,17 +800,14 @@ class Mark4Header(Mark4TrackHeader):
         for k in self.keys():
             v = self[k]
             if len(v) == 1:
-                outs.append('{0}: {1}'.format(
-                    k, hex(v[0]) if self._repr_as_hex(k) else v[0]))
+                repr_value = self._repr_value(k, v[0])
             elif np.all(v == v[0]):
-                outs.append('{0}: [{1}]*{2}'.format(
-                    k, hex(v[0]) if self._repr_as_hex(k) else v[0], v.size))
+                repr_value = f'[{self._repr_value(k, v[0])}]*{v.size}'
             else:
                 if len(v) > 4:
                     v = (v[0], '...', v[-1])
-                outs.append('{0}: [{1}]'.format(k, ', '.join(
-                    (hex(_v) if _v != '...' and self._repr_as_hex(k)
-                     else str(_v)) for _v in v)))
+                repr_value = '[{}]'.format(', '.join([self._repr_value(k, _v)
+                                                      for _v in v]))
+            outs.append(f'{k}: {repr_value}')
 
-        return "<{0} {1}>".format(name,
-                                  (",\n  " + len(name) * " ").join(outs))
+        return "<{} {}>".format(name, (",\n  " + " "*len(name)).join(outs))
