@@ -1,4 +1,6 @@
 # Licensed under the GPLv3 - see LICENSE
+import pickle
+
 import pytest
 import numpy as np
 from numpy.testing import assert_array_equal
@@ -6,14 +8,15 @@ import astropy.units as u
 from astropy.time import Time
 
 from ... import mark4
-from ...vlbi_base.base import HeaderNotFoundError
-from ...vlbi_base.encoding import OPTIMAL_2BIT_HIGH
+from ...base.base import HeaderNotFoundError
+from ...base.encoding import OPTIMAL_2BIT_HIGH
 from ..header import Mark4TrackHeader
 from ..payload import reorder32, reorder64
 from ...data import (SAMPLE_MARK4 as SAMPLE_FILE,
                      SAMPLE_MARK4_32TRACK as SAMPLE_32TRACK,
                      SAMPLE_MARK4_32TRACK_FANOUT2 as SAMPLE_32TRACK_FANOUT2,
-                     SAMPLE_MARK4_16TRACK as SAMPLE_16TRACK)
+                     SAMPLE_MARK4_16TRACK as SAMPLE_16TRACK,
+                     SAMPLE_MARK4_64TRACK_FANOUT2_FT as SAMPLE_64TRACK_FT)
 
 # Results from mark5access on 2015-JAN-22.
 # m5d evn/Ar/gp052d_ar_no0021 MKIV1_4-512-8-2 1000
@@ -70,8 +73,10 @@ class TestMark4:
         assert header.nbytes == 160 * 64 // 8
         assert header.fanout == 4
         assert header.bps == 2
+        assert not header.complex_data
         assert not np.all(~header['magnitude_bit'])
         assert header.nchan == 8
+        assert header.sample_shape == (8,)
         assert int(header.time.mjd) == 56824
         assert header.time.isot == '2014-06-16T07:38:12.47500'
         assert header.samples_per_frame == 20000 * 4
@@ -104,6 +109,13 @@ class TestMark4:
             system_id=108)
         assert header4 == header
         assert header4.mutable is True
+        # Check we can set the header to not complex
+        assert not header4.complex_data
+        header4.complex_data = False
+        assert not header4.complex_data
+        # But not to complex, since Mark 4 does not support that.
+        with pytest.raises(ValueError):
+            header4.complex_data = True
         # Check that passing a year into decade leads to an error.
         with pytest.raises(AssertionError):
             mark4.Mark4Header(header.words, decade=2014)
@@ -112,6 +124,26 @@ class TestMark4:
             mark4.Mark4Header.fromvalues(
                 ntrack=64, samples_per_frame=80001, bps=2, nsb=1,
                 time=header.time, system_id=108)
+        # And that data cannot be complex also by initializing from values.
+        with pytest.raises(ValueError, match='can only be set to False'):
+            mark4.Mark4Header.fromvalues(
+                ntrack=64, bps=2, complex_data=True, time=header.time)
+        # That fanout has to be 1, 2, or 4
+        with pytest.raises(ValueError, match='fanout=1, 2, or 4'):
+            mark4.Mark4Header.fromvalues(
+                ntrack=64, bps=2, time=header.time, fanout=8)
+        # That nsb should be 1 or 2.
+        with pytest.raises(ValueError, match='can only be 1 or 2'):
+            mark4.Mark4Header.fromvalues(
+                ntrack=64, bps=2, time=header.time, nsb=3)
+        # That we do not have track assignments for ntrack=8
+        with pytest.raises(ValueError, match='track assignments only'):
+            mark4.Mark4Header.fromvalues(ntrack=8, bps=2, fanout=1)
+        # And that bps=1, fanout=1 is not supported for nsb=2,
+        # since it needs track assignments.
+        # TODO: is this necessary??
+        with pytest.raises(ValueError, match='does not support bps=1'):
+            mark4.Mark4Header.fromvalues(ntrack=64, bps=1, fanout=1, nsb=2)
         # Check that passing approximate ref_time is equivalent to passing a
         # decade.
         with open(SAMPLE_FILE, 'rb') as fh:
@@ -220,6 +252,9 @@ class TestMark4:
         assert np.all((h0w ^ p) & m == 0)
         assert np.all((h1w ^ p) & m == 0)
 
+        with pytest.raises(ValueError, match='need to pass in'):
+            mark4.Mark4Header.invariant_pattern()
+
     @pytest.mark.parametrize(('unit_year', 'ref_time', 'decade'),
                              [(5, Time('2014:1:12:00:00'), 2010),
                               (5, Time('2009:362:19:27:33'), 2000),
@@ -294,7 +329,8 @@ class TestMark4:
                 mark4.Mark4Payload.fromfile(s, header)
         payload3 = mark4.Mark4Payload.fromdata(payload.data, header)
         assert payload3 == payload
-        payload4 = mark4.Mark4Payload(payload.words, nchan=8, bps=2, fanout=4)
+        payload4 = mark4.Mark4Payload(payload.words, sample_shape=(8,),
+                                      bps=2, fanout=4)
         assert payload4 == payload
         with pytest.raises(ValueError):
             # Wrong number of channels.
@@ -310,7 +346,8 @@ class TestMark4:
                                         header)
         with pytest.raises(ValueError):
             # Wrong encoded data type for implied number of tracks of 32.
-            mark4.Mark4Payload(payload.words, nchan=4, bps=2, fanout=4)
+            mark4.Mark4Payload(payload.words, sample_shape=(4,),
+                               bps=2, fanout=4)
         with pytest.raises(ValueError):
             # Not little-endian encoded data.
             mark4.Mark4Payload(payload.words.astype('>u8'), header)
@@ -360,6 +397,26 @@ class TestMark4:
         assert repr_fh.startswith('Mark4FileReader')
         assert 'ntrack=64, decade=2010, ref_time=None' in repr_fh
 
+    def test_binary_file_info(self):
+        with mark4.open(SAMPLE_FILE, 'rb') as fh1:
+            info1 = fh1.info
+            assert info1.format == 'mark4'
+            assert {'decade', 'ref_time'} == set(info1.missing)
+
+        with mark4.open(SAMPLE_FILE, 'rb', decade=2010) as fh2:
+            info2 = fh2.info
+            assert info2.format == 'mark4'
+            assert not info2.missing
+            assert info2.offset0 == 0xa88
+            assert abs(info2.frame_rate
+                       - 32 * u.MHz / info2.samples_per_frame) < 1 * u.nHz
+
+        with mark4.open(SAMPLE_FILE, 'rb', decade='2010') as fh3:
+            info3 = fh3.info
+            assert info3.format == 'mark4'
+            assert info3.offset0 == 0xa88
+            assert 'header0' in info3.errors
+
     def test_frame(self, tmpdir):
         with mark4.open(SAMPLE_FILE, 'rb', decade=2010, ntrack=64) as fh:
             fh.seek(0xa88)
@@ -387,8 +444,17 @@ class TestMark4:
         with open(str(tmpdir.join('test.m4')), 'w+b') as s:
             frame.tofile(s)
             s.seek(0)
-            frame2 = mark4.Mark4Frame.fromfile(s, ntrack=64, decade=2010)
-        assert frame2 == frame
+            frame2a = mark4.Mark4Frame.fromfile(s, ntrack=64, decade=2010)
+        assert frame2a == frame
+        test_file2b = str(tmpdir.join('test2.m4'))
+        with mark4.open(test_file2b, 'wb') as fw:
+            fw.write_frame(frame)
+            fw.write_frame(frame.data, ntrack=64, decade=2010, **frame.header)
+        with mark4.open(test_file2b, 'rb', ntrack=64, decade=2010) as fr:
+            frame2b = fr.read_frame()
+            frame2c = fr.read_frame()
+        assert frame2b == frame
+        assert frame2c == frame
         # Need frame.data here, since payload.data does not contain pieces
         # overwritten by header.
         frame3 = mark4.Mark4Frame.fromdata(frame.data, header)
@@ -444,7 +510,12 @@ class TestMark4:
         # Check scalar, including whether the shape is correct.
         assert frame[935].shape == data[935].shape
         assert np.all(frame[639] == data[639])
-        assert np.all(frame[640] == data[640])
+        assert np.all(frame[640, 3:] == data[640, 3:])
+        assert np.all(frame[-4, -1] == data[-4, -1])
+        with pytest.raises(IndexError, match='out of range'):
+            frame[100000000000]
+        with pytest.raises(TypeError, match='only be indexed or sliced'):
+            frame[[1, 2, 3]]
         # Check __setitem__.
         # Default frame read from a file is not mutable.
         with pytest.raises(ValueError):
@@ -794,6 +865,29 @@ class TestMark4:
                     < 1. * u.ns)
             assert fh._frame.header.decade == 2020
 
+    def test_pickle(self):
+        # Only simple tests here; more complete ones in vdif.
+        with mark4.open(SAMPLE_FILE, 'rs', ntrack=64, decade=2010,
+                        subset=0) as fh:
+            fh.seek(6)
+            pickled = pickle.dumps(fh)
+            fh.read(3)
+            with pickle.loads(pickled) as fh2:
+                assert fh2.tell() == 6
+                fh2.read(10)
+
+            assert fh.tell() == 9
+
+        with pickle.loads(pickled) as fh3:
+            assert fh3.tell() == 6
+            fh3.read(1)
+
+        closed = pickle.dumps(fh)
+        with pickle.loads(closed) as fh4:
+            assert fh4.closed
+            with pytest.raises(ValueError):
+                fh4.read(1)
+
     # Test that writing an incomplete stream is possible, and that frame set is
     # appropriately marked as invalid.
     @pytest.mark.parametrize('fill_value', (0., -999.))
@@ -803,8 +897,7 @@ class TestMark4:
             record = fr.read(10)
             with pytest.warns(UserWarning, match='partial buffer'):
                 with mark4.open(m4_incomplete, 'ws', header0=fr.header0,
-                                sample_rate=32*u.MHz,
-                                ntrack=64, decade=2010) as fw:
+                                sample_rate=32*u.MHz) as fw:
                     fw.write(record)
 
         with mark4.open(m4_incomplete, 'rs', sample_rate=32*u.MHz,
@@ -891,8 +984,16 @@ class TestMark4:
 class Test32TrackFanout4():
     def test_locate_frames(self):
         with mark4.open(SAMPLE_32TRACK, 'rb') as fh:
-            assert fh.locate_frames() == [9656, 9656+32*2500]
+            # Simple first: let frame_nbytes stand in for ntrack:
+            expected = [9656, 9656+32*2500]
+            assert fh.locate_frames(frame_nbytes=32*2500) == expected
+            assert fh.ntrack is None
+            # But if we pass in nothing, it autodeterminess
+            assert fh.locate_frames() == expected
             assert fh.ntrack == 32
+            # And frame_nbytes has to make sense
+            with pytest.raises(ValueError, match='multiple of 2500 bytes'):
+                fh.locate_frames(frame_nbytes=500)
 
     def test_header(self):
         with open(SAMPLE_32TRACK, 'rb') as fh:
@@ -1092,7 +1193,7 @@ def test_start_at_last_frame(tmpdir):
     frame_rate = sample_rate / 80000
     start_time = Time('2012-01-02') - 1/frame_rate
     with mark4.open(fl, 'ws', sample_rate=sample_rate, time=start_time,
-                    ntrack=32, nchan=4, fanout=4, bps=2) as fw:
+                    ntrack=32, sample_shape=(4,), fanout=4, bps=2) as fw:
         fw.write(np.ones((80000*2, 4)))
 
     with mark4.open(fl, 'rs', decade=2010) as fr:
@@ -1107,3 +1208,77 @@ def test_file_streamer_continuous(sample):
     with mark4.open(sample, 'rs', sample_rate=sample_rate, decade=2010) as fs:
         assert fs.info.readable
         assert 'no obvious gaps' in fs.info.checks['continuous']
+
+
+class Test64TrackFt():
+    def test_locate_frame(self):
+        with mark4.open(SAMPLE_64TRACK_FT, 'rb') as fh:
+            assert fh.locate_frames()[0] == 124288
+            assert fh.ntrack == 64
+
+    def test_header(self):
+        with open(SAMPLE_64TRACK_FT, 'rb') as fh:
+            fh.seek(124288)
+            header = mark4.Mark4Header.fromfile(fh, ntrack=64, decade=2010)
+
+        # Try initialising with properties instead of keywords.
+        # * time imply the decade, bcd_unit_year, bcd_day, bcd_hour,
+        #   bcd_minute, bcd_second, bcd_fraction;
+        # * ntrack, samples_per_frame, bps define headstack_id, bcd_track_id,
+        #   fan_out, and magnitude_bit;
+        # * explicitly set lsb_output and converter_id as they are so odd.
+        header1 = mark4.Mark4Header.fromvalues(
+            ntrack=64, samples_per_frame=40000,
+            time=header.time, system_id=114, lsb_output=header['lsb_output'],
+            converter_id=header['converter_id'],
+            magnitude_bit=header['magnitude_bit'])
+        assert header1 == header
+
+    def test_file_streamer(self, tmpdir):
+        with mark4.open(SAMPLE_64TRACK_FT, 'rs', sample_rate=32*u.MHz,
+                        ntrack=64, decade=2010) as fh:
+            header0 = fh.header0
+            assert fh.samples_per_frame == 40000
+            assert fh.sample_rate == 32 * u.MHz
+            start_time = fh.start_time
+            assert start_time.yday == '2019:128:17:32:21.07250'
+            record = fh.read(40000)
+            fh_raw_tell1 = fh.fh_raw.tell()
+            assert fh_raw_tell1 == 160000 + 124288
+            fh.fh_raw.seek(0)
+            preheader_junk = fh.fh_raw.read(124288)
+
+        assert np.all(record[:320] == 0.)
+        # Note: cannot directly compare with mark5access/m5d,
+        # since that has the wrong bit reordering!  Can only compare the
+        # second half of each 32 tracks, i.e., 4:8 and 12:16. Still, doing
+        # m5d sample_64track_fanout2.m4 MKIV1_2-128-16-2 1000, and
+        # taking first 28 payload samples:
+        m5access_data = np.array(
+            [[3, -3, -1, -3, 1, 1, 3, -3, -1, -3, 1, -1, -1, 1, 1, -1],
+             [3, -3, 1, 3, 1, 1, -1, 1, 3, -3, 1, 3, -1, 1, 3, 3],
+             [-3, 3, 1, -1, -1, -1, -3, 3, -3, 3, -1, 1, -3, -1, -1, 3],
+             [-1, 1, -1, -3, -1, 3, 3, 3, 1, 1, 1, 1, -1, -1, -3, -1]])
+        assert np.all(record[320:324, 4:8].astype(int)
+                      == m5access_data[:, 4:8])
+        assert np.all(record[320:324, 12:].astype(int)
+                      == m5access_data[:, 12:])
+
+        fl = str(tmpdir.join('test.m4'))
+        with mark4.open(fl, 'ws', header0=header0, sample_rate=32*u.MHz) as fw:
+            fw.fh_raw.write(preheader_junk)
+            fw.write(record)
+            number_of_bytes = fw.fh_raw.tell()
+            assert number_of_bytes == fh_raw_tell1
+
+        # Note: this test would not work if we wrote only a single record.
+        with mark4.open(fl, 'rs', sample_rate=32*u.MHz,
+                        ntrack=64, decade=2010) as fh:
+            assert fh.start_time == start_time
+            record2 = fh.read()
+            assert np.all(record2 == record)
+
+        with open(fl, 'rb') as fh, open(SAMPLE_64TRACK_FT, 'rb') as fr:
+            orig_bytes = fr.read(number_of_bytes)
+            conv_bytes = fh.read()
+            assert conv_bytes == orig_bytes
