@@ -7,7 +7,7 @@ https://github.com/demorest/mark5access.
 Also, for the Mark5B design, see
 https://www.haystack.mit.edu/tech/vlbi/mark5/mark5_memos/019.pdf
 """
-from .base import open  # noqa
+from .base import open, info  # noqa
 from .header import Mark5BHeader  # noqa
 from .payload import Mark5BPayload  # noqa
 from .frame import Mark5BFrame  # noqa
