@@ -1,12 +1,14 @@
 # Licensed under the GPLv3 - see LICENSE
+import pickle
+
 import pytest
 import numpy as np
 import astropy.units as u
 from astropy.time import Time
 
 from ... import mark5b
-from ...vlbi_base.encoding import OPTIMAL_2BIT_HIGH
-from ...vlbi_base.base import HeaderNotFoundError
+from ...base.encoding import OPTIMAL_2BIT_HIGH
+from ...base.base import HeaderNotFoundError
 from ...data import SAMPLE_MARK5B as SAMPLE_FILE
 
 
@@ -57,6 +59,7 @@ class TestMark5B:
         with open(SAMPLE_FILE, 'rb') as fh:
             header = mark5b.Mark5BHeader.fromfile(fh, kday=56000)
         assert header.nbytes == 16
+        assert not header.complex_data
         assert header.kday == 56000.
         assert header.jday == 821
         mjd, frac = divmod(header.time.mjd, 1)
@@ -93,10 +96,13 @@ class TestMark5B:
         header6.time == header.time
         header6.payload_nbytes = 10000
         header6.frame_nbytes = 10016
-        with pytest.raises(ValueError):
+        header6.complex_data = False
+        with pytest.raises(ValueError, match="'payload_nbytes'.*set to 10000"):
             header6.payload_nbytes = 9999
         with pytest.raises(ValueError):
             header6.frame_nbytes = 20
+        with pytest.raises(ValueError):
+            header6.complex_data = True
         # Regression tests
         header7 = header.copy()
         assert header7 == header  # This checks header.words
@@ -152,7 +158,8 @@ class TestMark5B:
     def test_payload(self, tmpdir):
         with open(SAMPLE_FILE, 'rb') as fh:
             fh.seek(16)  # Skip header.
-            payload = mark5b.Mark5BPayload.fromfile(fh, nchan=8, bps=2)
+            payload = mark5b.Mark5BPayload.fromfile(
+                fh, sample_shape=(8,), bps=2)
         assert payload._nbytes == 10000
         assert payload.nbytes == 10000
         assert payload.shape == (5000, 8)
@@ -170,20 +177,23 @@ class TestMark5B:
             payload.tofile(s)
             s.seek(0)
             payload2 = mark5b.Mark5BPayload.fromfile(
-                s, payload.sample_shape.nchan, payload.bps)
+                s, sample_shape=payload.sample_shape, bps=payload.bps)
             assert payload2 == payload
             with pytest.raises(EOFError):
                 # Too few bytes.
                 s.seek(100)
-                mark5b.Mark5BPayload.fromfile(s, payload.sample_shape.nchan,
-                                              payload.bps)
+                mark5b.Mark5BPayload.fromfile(
+                    s, sample_shape=payload.sample_shape, bps=payload.bps)
 
         payload3 = mark5b.Mark5BPayload.fromdata(payload.data, bps=payload.bps)
         assert payload3 == payload
         # Complex data should fail.
         with pytest.raises(ValueError):
-            mark5b.Mark5BPayload(payload3.words, complex_data=True)
-        with pytest.raises(ValueError):
+            mark5b.Mark5BPayload(payload3.words, sample_shape=(1,),
+                                 complex_data=True)
+        with pytest.raises(ValueError, match='encoded data should have len'):
+            mark5b.Mark5BPayload(payload3.words[:-2], sample_shape=(1,))
+        with pytest.raises(ValueError, match='complex'):
             mark5b.Mark5BPayload.fromdata(np.zeros((5000, 8), np.complex64),
                                           bps=2)
 
@@ -194,10 +204,12 @@ class TestMark5B:
     def test_payload_getitem_setitem(self, item):
         with open(SAMPLE_FILE, 'rb') as fh:
             fh.seek(16)  # Skip header.
-            payload = mark5b.Mark5BPayload.fromfile(fh, nchan=8, bps=2)
+            payload = mark5b.Mark5BPayload.fromfile(fh, sample_shape=(8,),
+                                                    bps=2)
         sel_data = payload.data[item]
         assert np.all(payload[item] == sel_data)
-        payload2 = mark5b.Mark5BPayload(payload.words.copy(), nchan=8, bps=2)
+        payload2 = mark5b.Mark5BPayload(payload.words.copy(),
+                                        sample_shape=(8,), bps=2)
         assert payload2 == payload
         payload2[item] = -sel_data
         check = payload.data
@@ -258,18 +270,24 @@ class TestMark5B:
             for key in ('sample_rate', 'samples_per_frame', 'sample_shape',
                         'start_time'):
                 assert getattr(info, key) is None
+            # Only with a change in parent properties should a new info be
+            # generated (really a bit of an implementation detail).
+            info2 = fh.info
+            assert info is info2
             fh.nchan = 8
-            info = fh.info
-            assert set(info.missing.keys()) == {'ref_time', 'kday'}
+            info3 = fh.info
+            assert info3 is not info
+            assert set(info3.missing.keys()) == {'ref_time', 'kday'}
             for key in ('format', 'frame_rate', 'bps', 'complex_data',
                         'sample_rate', 'samples_per_frame', 'sample_shape'):
-                assert getattr(info, key) == expected[key]
-            assert info.start_time is None
+                assert getattr(info3, key) == expected[key]
+            assert info3.start_time is None
             fh.kday = 56000
-            info = fh.info
-            assert info.missing == {}
+            info4 = fh.info
+            assert info4 is not info3
+            assert info4.missing == {}
             for key, value in expected.items():
-                assert getattr(info, key) == value
+                assert getattr(info4, key) == value
 
             # Also check we cannot set, but can delete info
             with pytest.raises(AttributeError):
@@ -278,8 +296,15 @@ class TestMark5B:
             assert 'info' in fh.__dict__
             del fh.info
             assert 'info' not in fh.__dict__
+            info5 = fh.info
+            assert info5 is not info4
+            assert info5.missing == {}
+            for key, value in expected.items():
+                assert getattr(info5, key) == value
 
-        assert 'closed' in repr(fh.info)
+        info6 = fh.info
+        assert info6 is not info5
+        assert 'closed' in repr(info6)
 
         # Finally, also do a brief check of the stream reader info.
         # Note that the readers properties themselves are tested further below.
@@ -309,10 +334,24 @@ class TestMark5B:
 
         assert info() == stream_expected
 
+    def test_binary_file_info_invalid_data(self):
+        with mark5b.open(SAMPLE_FILE, 'rb', kday='56000') as fh:
+            info = fh.info
+            assert info.format == 'mark5b'
+            assert set(info.missing) == {'nchan'}
+            assert 'header0' in info.errors
+
+        with mark5b.open(SAMPLE_FILE, 'rb', ref_time='56000', nchan=8) as fh:
+            info = fh.info
+            assert info.format == 'mark5b'
+            assert not info.missing
+            assert 'header0' in info.errors
+
     def test_frame(self, tmpdir):
         with mark5b.open(SAMPLE_FILE, 'rb', kday=56000, nchan=8, bps=2) as fh:
             header = mark5b.Mark5BHeader.fromfile(fh, kday=56000)
-            payload = mark5b.Mark5BPayload.fromfile(fh, nchan=8, bps=2)
+            payload = mark5b.Mark5BPayload.fromfile(fh, sample_shape=(8,),
+                                                    bps=2)
             fh.seek(0)
             frame = fh.read_frame()
 
@@ -329,9 +368,9 @@ class TestMark5B:
         with open(str(tmpdir.join('test.m5b')), 'w+b') as s:
             frame.tofile(s)
             s.seek(0)
-            frame2 = mark5b.Mark5BFrame.fromfile(s, kday=56000,
-                                                 nchan=frame.shape[1],
-                                                 bps=frame.payload.bps)
+            frame2 = mark5b.Mark5BFrame.fromfile(
+                s, kday=56000, sample_shape=frame.sample_shape,
+                bps=frame.payload.bps)
         assert frame2 == frame
 
         # Check passing in reference time.
@@ -362,9 +401,10 @@ class TestMark5B:
         with open(str(tmpdir.join('test8.m5b')), 'w+b') as s:
             frame8.tofile(s)
             s.seek(0)
-            frame9 = mark5b.Mark5BFrame.fromfile(s, kday=56000,
-                                                 nchan=frame8.shape[1],
-                                                 bps=frame8.payload.bps)
+            frame9 = mark5b.Mark5BFrame.fromfile(
+                s, kday=56000,
+                sample_shape=frame8.sample_shape,
+                bps=frame8.payload.bps)
         assert frame9.valid is False
         assert np.all(frame9.data == 0.)
         assert np.all(frame9.payload.words == 0x11223344)
@@ -421,11 +461,23 @@ class TestMark5B:
                         frame_rate=frame_rate)
         assert abs(header.get_time(frame_rate)
                    - start_time - 25599. / frame_rate) < 1. * u.ns
-        # Check rounding to the nearest second when less than 2 ns away.
-        header.set_time(time=(start_time + 0.9 * u.ns), frame_rate=frame_rate)
+        # Check rounding to the nearest second when less than 2 ns away,
+        # without frame_rate
+        header.set_time(time=(start_time + 0.9 * u.ns))
         assert header.seconds == header0.seconds
-        header.set_time(time=(start_time - 0.9 * u.ns), frame_rate=frame_rate)
+        header.set_time(time=(start_time - 0.9 * u.ns))
         assert header.seconds == header0.seconds
+        # And also proper rounding for larger time differences if a
+        # frame rate is passed in.
+        header.set_time(start_time + 0.4/frame_rate, frame_rate=frame_rate)
+        assert header.seconds == header0.seconds
+        assert header['frame_nr'] == 0
+        header.set_time(start_time - 0.4/frame_rate, frame_rate=frame_rate)
+        assert header.seconds == header0.seconds
+        assert header['frame_nr'] == 0
+
+        with pytest.raises(ValueError, match='cannot calculate frame rate'):
+            header.set_time(time=start_time+1./frame_rate)
 
     def test_locate_frames(self, tmpdir):
         with mark5b.open(SAMPLE_FILE, 'rb', kday=56000) as fh:
@@ -750,11 +802,49 @@ class TestMark5B:
             assert abs(fh.start_time - test_time) < 1.*u.ns
             assert fh.header0['frame_nr'] == 198
 
+        # Check that we can fail sample rate determination if we have only
+        # a single frame.  In the process, might as well check both versions
+        # of write_frame...
+        with mark5b.open(m5_test_samplerate, 'wb') as fw:
+            fw.write_frame(frame)
+            fw.write_frame(frame.data, frame.header, bps=2)
+
+        with mark5b.open(m5_test_samplerate, 'rb', nchan=8) as fr:
+            data1 = fr.read_frame()
+            data2 = fr.read_frame()
+            assert np.all(data1 == data2)
+
+            with pytest.raises(EOFError, match='can also not'):
+                fr.get_frame_rate()
+
     def test_filestreamer_readable(self):
         with mark5b.open(SAMPLE_FILE, 'rs', sample_rate=32*u.MHz,
                          ref_time=Time('2015-01-01'), nchan=8, bps=2) as fh:
             assert fh.info.readable
 
+    def test_pickle(self):
+        # Only simple tests here; more complete ones in vdif.
+        with mark5b.open(SAMPLE_FILE, 'rs', sample_rate=32*u.MHz,
+                         ref_time=Time('2015-01-01'), nchan=8, bps=2) as fh:
+            fh.seek(6)
+            pickled = pickle.dumps(fh)
+            fh.read(3)
+            with pickle.loads(pickled) as fh2:
+                assert fh2.tell() == 6
+                fh2.read(10)
+
+            assert fh.tell() == 9
+
+        with pickle.loads(pickled) as fh3:
+            assert fh3.tell() == 6
+            fh3.read(1)
+
+        closed = pickle.dumps(fh)
+        with pickle.loads(closed) as fh4:
+            assert fh4.closed
+            with pytest.raises(ValueError):
+                fh4.read(1)
+
     def test_sequentialfile(self, tmpdir):
         """Tests writing and reading of sequential files.
 
