@@ -6,17 +6,19 @@ import numpy as np
 import astropy.units as u
 from astropy.utils import lazyproperty
 
-from ..vlbi_base.base import (make_opener, VLBIFileBase, VLBIFileReaderBase,
-                              VLBIStreamBase, VLBIStreamReaderBase,
-                              VLBIStreamWriterBase, HeaderNotFoundError)
+from ..base.base import (
+    FileBase, VLBIFileReaderBase,
+    VLBIStreamReaderBase, StreamWriterBase,
+    FileOpener, FileInfo, HeaderNotFoundError)
 from .header import VDIFHeader
 from .payload import VDIFPayload
 from .frame import VDIFFrame, VDIFFrameSet
 from .file_info import VDIFFileReaderInfo
 
 
-__all__ = ['VDIFFileReader', 'VDIFFileWriter', 'VDIFStreamBase',
-           'VDIFStreamReader', 'VDIFStreamWriter', 'open']
+__all__ = ['VDIFFileReader', 'VDIFFileWriter',
+           'VDIFStreamBase', 'VDIFStreamReader', 'VDIFStreamWriter',
+           'open', 'info']
 
 # Check code on 2015-MAY-30
 # 00000000  77 2c db 00 00 00 00 1c  75 02 00 20 fc ff 01 04  # header 0 - 3
@@ -158,9 +160,8 @@ class VDIFFileReader(VLBIFileReaderBase):
         try:
             return super().get_frame_rate()
         except Exception as exc:
-            with self.temporary_offset():
+            with self.temporary_offset(0):
                 try:
-                    self.seek(0)
                     header = self.read_header()
                     return (header.sample_rate
                             / header.samples_per_frame).to(u.Hz).round()
@@ -207,7 +208,7 @@ class VDIFFileReader(VLBIFileReaderBase):
                         n_check -= 1
             except EOFError:
                 # Hack: let through very short files (like our samples).
-                if self.seek(0, 2) > ((check+2) * len(thread_ids)
+                if self.seek(0, 2) > (check * len(thread_ids)
                                       * header0.frame_nbytes):
                     raise
 
@@ -265,7 +266,7 @@ class VDIFFileReader(VLBIFileReaderBase):
 
         Raises
         ------
-        ~baseband.vlbi_base.base.HeaderNotFoundError
+        ~baseband.base.base.HeaderNotFoundError
             If no header could be located.
         AssertionError
             If the header did not pass verification.
@@ -313,7 +314,7 @@ class VDIFFileReader(VLBIFileReaderBase):
         raise HeaderNotFoundError("could not locate a nearby header.")
 
 
-class VDIFFileWriter(VLBIFileBase):
+class VDIFFileWriter(FileBase):
     """Simple writer for VDIF files.
 
     Adds `write_frame` and `write_frameset` methods to the basic VLBI
@@ -362,22 +363,11 @@ class VDIFFileWriter(VLBIFileBase):
         return data.tofile(self.fh_raw)
 
 
-class VDIFStreamBase(VLBIStreamBase):
-    """Base for VDIF streams."""
+class VDIFStreamBase:
+    """Provides sample shape maker and fast time and index getting/setting."""
 
     _sample_shape_maker = namedtuple('SampleShape', 'nthread, nchan')
 
-    def __init__(self, fh_raw, header0, sample_rate=None, nthread=1,
-                 squeeze=True, subset=(), fill_value=0., verify=True):
-        samples_per_frame = header0.samples_per_frame
-
-        super().__init__(
-            fh_raw=fh_raw, header0=header0, sample_rate=sample_rate,
-            samples_per_frame=samples_per_frame,
-            unsliced_shape=(nthread, header0.nchan), bps=header0.bps,
-            complex_data=header0['complex_data'], squeeze=squeeze,
-            subset=subset, fill_value=fill_value, verify=verify)
-
     def _get_time(self, header):
         """Get time from a header.
 
@@ -386,12 +376,33 @@ class VDIFStreamBase(VLBIStreamBase):
         """
         return header.get_time(frame_rate=self._frame_rate)
 
+    def _set_time(self, header, time):
+        """Set time in a header.
+
+        This passes on sample rate, which not all VDIF headers can calculate.
+        """
+        header.update(time=time, frame_rate=self._frame_rate)
+
+    def _get_index(self, header):
+        # Override to avoid explicit time calculations.
+        return int(round((header['seconds'] - self.header0['seconds'])
+                         * self._frame_rate.to_value(u.Hz)
+                         + header['frame_nr'] - self.header0['frame_nr']))
+
+    def _set_index(self, header, index):
+        # Override to avoid explicit time calculations.
+        dt, frame_nr = divmod(index + self.header0['frame_nr'],
+                              int(round(self._frame_rate.to_value(u.Hz))))
+        seconds = self.header0['seconds'] + dt
+        header['seconds'] = seconds
+        header['frame_nr'] = frame_nr
+
     def __repr__(self):
         return ("<{s.__class__.__name__} name={s.name} offset={s.offset}\n"
                 "    sample_rate={s.sample_rate},"
                 " samples_per_frame={s.samples_per_frame},\n"
                 "    sample_shape={s.sample_shape},\n"
-                "    bps={h.bps}, complex_data={s.complex_data},"
+                "    bps={s.bps}, complex_data={s.complex_data},"
                 " edv={h.edv}, station={h.station},\n"
                 "    {sub}start_time={s.start_time}>"
                 .format(s=self, h=self.header0,
@@ -441,8 +452,8 @@ class VDIFStreamReader(VDIFStreamBase, VLBIStreamReaderBase):
         nthread = len(thread_ids)
         super().__init__(
             fh_raw, header0, sample_rate=sample_rate,
-            nthread=nthread, squeeze=squeeze, subset=subset,
-            fill_value=fill_value, verify=verify)
+            sample_shape=(nthread, header0.nchan), squeeze=squeeze,
+            subset=subset, fill_value=fill_value, verify=verify)
         self._raw_offsets.frame_nbytes *= nthread
 
         # Check whether we are reading only some threads.  This is somewhat
@@ -482,17 +493,20 @@ class VDIFStreamReader(VDIFStreamBase, VLBIStreamReaderBase):
     def _last_header(self):
         """Last header of the file."""
         # Go to end of file.
-        with self.fh_raw.temporary_offset() as fh_raw:
-            maximum = 2 * self._raw_offsets.frame_nbytes
+        maximum = 2 * self._raw_offsets.frame_nbytes
+        with self.fh_raw.temporary_offset(
+                -self.header0.frame_nbytes, 2) as fh_raw:
             # Find first header with same thread_id going backward.
-            fh_raw.seek(-self.header0.frame_nbytes, 2)
             locations = fh_raw.locate_frames(self.header0, forward=False,
                                              maximum=maximum, check=(-1, 1))
             for location in locations:
                 fh_raw.seek(location)
                 try:
                     header = fh_raw.read_header(edv=self.header0.edv)
-                except Exception:
+                except Exception:  # pragma: no cover
+                    # If reading fails, just try the next one -- we would
+                    # use loose a bit of the end of the file.  Would require
+                    # an EDV verify stricter than what locate_frames uses.
                     continue
 
                 if header['thread_id'] == self.header0['thread_id']:
@@ -519,16 +533,11 @@ class VDIFStreamReader(VDIFStreamBase, VLBIStreamReaderBase):
                                          edv=self.header0.edv,
                                          verify=self.verify)
 
-    def _tell_frame(self, frame):
-        return int(round((frame['seconds'] - self.header0['seconds'])
-                         * self._frame_rate.to_value(u.Hz)
-                         + frame['frame_nr'] - self.header0['frame_nr']))
-
     def _bad_frame(self, index, frameset, exc):
         # Duplication of base class, but able to deal with missing
         # frames inside a frame set.
-        if (frameset is not None and self._tell_frame(frameset) == index
-                and index == self._tell_frame(self._last_header)):
+        if (frameset is not None and self._get_index(frameset) == index
+                and index == self._get_index(self._last_header)):
             # If we got an exception because we're trying to read beyond the
             # last frame, the frame is almost certainly OK, so keep it.
             return frameset
@@ -565,7 +574,7 @@ class VDIFStreamReader(VDIFStreamBase, VLBIStreamReaderBase):
             raise exc
 
         # Don't yet know how to deal with excess data.
-        header_index = self._tell_frame(header)
+        header_index = self._get_index(header)
         if header_index < index:
             exc.args += (msg + ' There appears to be excess data.',)
             raise exc
@@ -590,7 +599,7 @@ class VDIFStreamReader(VDIFStreamBase, VLBIStreamReaderBase):
                 exc.args += (msg + ' Could not find previous index.',)
                 raise exc
 
-            header_index = self._tell_frame(header)
+            header_index = self._get_index(header)
             if header_index < header1_index:
                 # While we are at it: if we pass an index boundary,
                 # update the list of known indices.
@@ -613,10 +622,7 @@ class VDIFStreamReader(VDIFStreamBase, VLBIStreamReaderBase):
             # (after the very long else clause).
             # TODO: just use the writer's _make_frame??
             frames = {}
-            dt, frame_nr = divmod(index + self.header0['frame_nr'],
-                                  int(self._frame_rate.to_value(u.Hz)))
-            header['frame_nr'] = frame_nr
-            header['seconds'] = self.header0['seconds'] + dt
+            self._set_index(header, index)
         else:
             assert header1_index == index, \
                 'at this point, we should have a good header.'
@@ -710,7 +716,7 @@ class VDIFStreamReader(VDIFStreamBase, VLBIStreamReaderBase):
             # If the next header is of the next frame, set up the raw
             # offset (which likely will be needed, saving some time).
             if (next_header is not None
-                    and self._tell_frame(next_header) == index + 1):
+                    and self._get_index(next_header) == index + 1):
                 self._raw_offsets[index+1] = self.fh_raw.tell()
 
         # Create invalid frame template,
@@ -740,7 +746,7 @@ class VDIFStreamReader(VDIFStreamBase, VLBIStreamReaderBase):
         return frameset
 
 
-class VDIFStreamWriter(VDIFStreamBase, VLBIStreamWriterBase):
+class VDIFStreamWriter(VDIFStreamBase, StreamWriterBase):
     """VLBI VDIF format writer.
 
     Encodes and writes sequences of samples to file.
@@ -750,8 +756,7 @@ class VDIFStreamWriter(VDIFStreamBase, VLBIStreamWriterBase):
     fh_raw : filehandle
         Which will write filled sets of frames to storage.
     header0 : :class:`~baseband.vdif.VDIFHeader`
-        Header for the first frame, holding time information, etc.  Can instead
-        give keyword arguments to construct a header (see ``**kwargs``).
+        Header for the first frame, holding time information, etc.
     sample_rate : `~astropy.units.Quantity`
         Number of complete samples per second, i.e. the rate at which each
         channel in each thread is sampled.  For EDV 1 and 3, can
@@ -761,79 +766,39 @@ class VDIFStreamWriter(VDIFStreamBase, VLBIStreamWriterBase):
     squeeze : bool, optional
         If `True` (default), `write` accepts squeezed arrays as input, and
         adds any dimensions of length unity.
-    **kwargs
-        If no header is given, an attempt is made to construct one from these.
-        For a standard header, this would include the following.
-
-    --- Header keywords : (see :meth:`~baseband.vdif.VDIFHeader.fromvalues`)
-
-    time : `~astropy.time.Time`
-        Start time of the file.  Can instead pass on ``ref_epoch`` and
-        ``seconds``.
-    nchan : int, optional
-        Number of channels (default: 1).  Note: different numbers of channels
-        per thread is not supported.
-    complex_data : bool, optional
-        Whether data are complex.  Default: `False`.
-    bps : int, optional
-        Bits per elementary sample, i.e. per real or imaginary component for
-        complex data.  Default: 1.
-    samples_per_frame : int
-        Number of complete samples per frame.  Can alternatively use
-        ``frame_length``, the number of 8-byte words for header plus payload.
-        For some EDV, this number is fixed (e.g., ``frame_length=629`` for
-        ``edv=3``, which corresponds to 20000 real 2-bit samples per frame).
-    station : 2 characters, optional
-        Station ID.  Can also be an unsigned 2-byte integer.  Default: 0.
-    edv : {`False`, 0, 1, 2, 3, 4, 0xab}
-        Extended Data Version.
     """
 
     def __init__(self, fh_raw, header0=None, sample_rate=None, nthread=1,
-                 squeeze=True, **kwargs):
+                 squeeze=True):
         fh_raw = VDIFFileWriter(fh_raw)
-        if header0 is None:
-            if sample_rate is not None:
-                kwargs['sample_rate'] = sample_rate
-            header0 = VDIFHeader.fromvalues(**kwargs)
+        # Get header sample rate
+        try:
+            header_sample_rate = header0.sample_rate
+        except AttributeError:
+            header_sample_rate = None
 
-        # If header was passed but not sample_rate, extract sample_rate.
         if sample_rate is None:
-            try:
-                sample_rate = header0.sample_rate
-            except AttributeError:
+            if header_sample_rate is None:
                 raise ValueError("the sample rate must be passed either "
                                  "explicitly, or through the header if it "
                                  "can be stored there.")
 
-        super().__init__(fh_raw, header0, sample_rate=sample_rate,
-                         nthread=nthread, squeeze=squeeze)
-        # Set sample rate in the header, if it's possible, and not set already.
-        try:
-            header_sample_rate = self.header0.sample_rate
-        except AttributeError:
-            pass
-        else:
-            if header_sample_rate == 0:
-                self.header0.sample_rate = self.sample_rate
-            assert self.header0.sample_rate == self.sample_rate
+            sample_rate = header_sample_rate
+        elif header_sample_rate is not None:
+            assert sample_rate == header_sample_rate, (
+                'sample_rate on header inconsistent with that passed in.')
 
-        self._frameset = VDIFFrameSet.fromdata(
-            np.zeros((self.samples_per_frame,) + self._unsliced_shape,
+        super().__init__(fh_raw, header0, sample_rate=sample_rate,
+                         sample_shape=(nthread, header0.nchan),
+                         squeeze=squeeze)
+
+        self._frame = VDIFFrameSet.fromdata(
+            np.zeros((self.samples_per_frame, nthread, header0.nchan),
                      dtype=np.complex64 if self.complex_data else np.float32),
             self.header0)
 
-    def _make_frame(self, index):
-        dt, frame_nr = divmod(index + self.header0['frame_nr'],
-                              int(self._frame_rate.to_value(u.Hz)))
-        seconds = self.header0['seconds'] + dt
-        # Reuse frameset.
-        self._frameset['seconds'] = seconds
-        self._frameset['frame_nr'] = frame_nr
-        return self._frameset
 
-
-open = make_opener('VDIF', globals(), doc="""
+open = FileOpener.create(globals(), doc="""
 --- For reading a stream : (see :class:`~baseband.vdif.base.VDIFStreamReader`)
 
 sample_rate : `~astropy.units.Quantity`, optional
@@ -873,9 +838,31 @@ file_size : int or None, optional
     If `None` (default), the file size is unlimited, and only the first
     file will be written to.
 **kwargs
-    If the header is not given, an attempt will be made to construct one
-    with any further keyword arguments.  See
-    :class:`~baseband.vdif.base.VDIFStreamWriter`.
+    If no header is given, an attempt is made to construct one from these.
+    For a standard header, this would include the following.
+
+--- Header keywords : (see :meth:`~baseband.vdif.VDIFHeader.fromvalues`)
+
+time : `~astropy.time.Time`
+    Start time of the file.  Can instead pass on ``ref_epoch`` and
+    ``seconds``.
+nchan : int, optional
+    Number of channels (default: 1).  Note: different numbers of channels
+    per thread is not supported.
+complex_data : bool, optional
+    Whether data are complex.  Default: `False`.
+bps : int, optional
+    Bits per elementary sample, i.e. per real or imaginary component for
+    complex data.  Default: 1.
+samples_per_frame : int
+    Number of complete samples per frame.  Can alternatively use
+    ``frame_length``, the number of 8-byte words for header plus payload.
+    For some EDV, this number is fixed (e.g., ``frame_length=629`` for
+    ``edv=3``, which corresponds to 20000 real 2-bit samples per frame).
+station : 2 characters, optional
+    Station ID.  Can also be an unsigned 2-byte integer.  Default: 0.
+edv : {`False`, 0, 1, 2, 3, 4, 0xab}
+    Extended Data Version.
 
 Notes
 -----
@@ -886,3 +873,6 @@ written to.  One may also pass in a `~baseband.helpers.sequentialfile` object
 (opened in 'rb' mode for reading or 'w+b' for writing), though for typical use
 cases it is practically identical to passing in a list or template.
 """)
+
+
+info = FileInfo.create(globals())
