@@ -10,7 +10,7 @@ For the VDIF specification, see https://www.vlbi.org/vdif
 """
 import numpy as np
 
-from ..vlbi_base.frame import VLBIFrameBase
+from ..base.frame import FrameBase
 from .header import VDIFHeader, VDIFBaseHeader
 from .payload import VDIFPayload
 
@@ -18,7 +18,7 @@ from .payload import VDIFPayload
 __all__ = ['VDIFFrame', 'VDIFFrameSet']
 
 
-class VDIFFrame(VLBIFrameBase):
+class VDIFFrame(FrameBase):
     """Representation of a VDIF data frame, consisting of a header and payload.
 
     Parameters
@@ -28,9 +28,9 @@ class VDIFFrame(VLBIFrameBase):
         header information.
     payload : `~baseband.vdif.VDIFPayload`
         Wrapper around the payload, provding mechanisms to decode it.
-    valid : bool or None
-        Whether the data are valid.  If `None` (default), is inferred from
-        header.  Note that ``header`` is changed in-place if `True` or `False`.
+    valid : bool, optional
+        Whether the data are valid.  Default: inferred from header.
+        Note that ``header`` is changed in-place if `True` or `False`.
     verify : bool
         Whether or not to do basic assertions that check the integrity
         (e.g., that channel information and whether or not data are complex
@@ -65,14 +65,6 @@ class VDIFFrame(VLBIFrameBase):
     _header_class = VDIFHeader
     _payload_class = VDIFPayload
 
-    def __init__(self, header, payload, valid=None, verify=True):
-        self.header = header
-        self.payload = payload
-        if valid is not None:
-            self.valid = valid
-        if verify:
-            self.verify()
-
     def verify(self):
         """Verify integrity.
 
@@ -80,7 +72,7 @@ class VDIFFrame(VLBIFrameBase):
         data shape and type.
         """
         super().verify()
-        assert self.header['complex_data'] == (self.payload.dtype.kind == 'c')
+        assert self.header.complex_data == (self.payload.dtype.kind == 'c')
         assert self.payload.shape == (self.header.samples_per_frame,
                                       self.header.nchan)
 
@@ -119,30 +111,6 @@ class VDIFFrame(VLBIFrameBase):
         # using header, frame verification is unnecessary.
         return cls(header, payload, verify=False)
 
-    @classmethod
-    def fromdata(cls, data, header=None, verify=True, **kwargs):
-        """Construct frame from data and header.
-
-        Parameters
-        ----------
-        data : `~numpy.ndarray`
-            Array holding complex or real data to be encoded.
-        header : `~baseband.vdif.VDIFHeader` or None
-            If not given, will attempt to generate one using the keywords.
-        verify : bool, optional
-            Whether or not to do basic assertions that check the integrity
-            (e.g., that channel information and whether or not data are complex
-            are consistent between header and data). Default: `True`.
-        **kwargs
-            If ``header`` is not given, these are used to initialize one.
-        """
-        if header is None:
-            header = cls._header_class.fromvalues(verify=verify, **kwargs)
-
-        payload = cls._payload_class.fromdata(data, header=header)
-
-        return cls(header, payload, verify=True)
-
     @classmethod
     def from_mark5b_frame(cls, mark5b_frame, verify=True, **kwargs):
         """Construct an Mark5B over VDIF frame (EDV=0xab).
@@ -311,7 +279,7 @@ class VDIFFrameSet:
                 header['thread_id'] = thread_id
                 headers.append(header)
 
-        frames = [VDIFFrame.fromdata(d, h, verify)
+        frames = [VDIFFrame.fromdata(d, h, verify=verify)
                   for d, h in zip(data.transpose(1, 0, 2), headers)]
         return cls(frames)
 
@@ -412,7 +380,7 @@ class VDIFFrameSet:
 
         single_sample = not isinstance(item[0], slice)
         if len(item) == 1:
-            return self.frames, False, item[0], single_sample, False
+            return self.frames, item[0], single_sample, False, False
 
         single_channel = (len(item) > 2
                           and np.empty(self.shape[2:])[item[2:]].ndim == 0)
@@ -455,8 +423,8 @@ class VDIFFrameSet:
             swapped = data.swapaxes(0, 1)
 
         swapped[0] = data0
-        for frame, frame_data in zip(frames[1:], swapped[1:]):
-            frame_data[...] = frame[frame_item]
+        for i, frame in enumerate(frames[1:]):
+            swapped[i+1] = frame[frame_item]
         return data
 
     def __setitem__(self, item, data):
@@ -485,7 +453,7 @@ class VDIFFrameSet:
 
         data = np.asanyarray(data)
         if single_channel:
-            if single_sample or data.ndim <= 1:
+            if single_sample and data.ndim <= 1:
                 swapped = np.broadcast_to(data, (len(frames),))
             else:
                 new_shape = (data.shape[0], len(frames))
@@ -517,7 +485,7 @@ class VDIFFrameSet:
         if attr in self.header0._properties:
             if attr in VDIFBaseHeader._properties:
                 return getattr(self.header0, attr)
-            values = np.array([getattr(f.header, attr) for f in self.frames])
+            values = np.hstack([getattr(f.header, attr) for f in self.frames])
             return values[0] if len(np.unique(values)) == 1 else values
         else:
             return self.__getattribute__(attr)
