@@ -1,6 +1,7 @@
 # Licensed under the GPLv3 - see LICENSE
 from copy import copy
 import io
+import pickle
 from collections import namedtuple
 
 import pytest
@@ -10,7 +11,7 @@ import astropy.units as u
 from ..header import HeaderParser, VLBIHeaderBase, four_word_struct
 from ..payload import VLBIPayloadBase
 from ..frame import VLBIFrameBase
-from ..base import (VLBIFileBase, VLBIStreamBase,
+from ..base import (VLBIFileBase, VLBIFileReaderBase, VLBIStreamBase,
                     VLBIStreamReaderBase, VLBIStreamWriterBase)
 
 
@@ -50,6 +51,9 @@ class TestVLBIBase:
             _struct = four_word_struct
             _header_parser = self.header_parser
             payload_nbytes = 8
+            bps = 8
+            sample_shape = (2,)
+            complex_data = False
 
         self.Header = Header
         self.header = self.Header([0x12345678, 0xffff0000, 0x0, 0xffffffff])
@@ -476,26 +480,84 @@ class TestVLBIBase:
             assert fh.closed
             assert fh.fh_raw.closed
         with io.open(filename, 'rb') as fr:
-            fh = VLBIFileBase(fr)
+            fh = VLBIFileReaderBase(fr)
             assert fh.fh_raw is fr
             assert fh.readable()
             assert not fh.writable()
             assert not fh.closed
             with pytest.raises(AttributeError):
                 fh.bla
-            assert repr(fh).startswith('VLBIFileBase(fh_raw')
+            assert repr(fh).startswith('VLBIFileReaderBase(fh_raw')
             assert fh.read() == b'abcd'
             fh.close()
             assert fh.closed
             assert fh.fh_raw.closed
 
+    def test_temporary_offset(self, tmpdir):
+        filename = str(tmpdir.join('test.dat'))
+        with io.open(filename, 'wb') as fw:
+            fw.write(b'abcdefghijklmnopqrstuvwxyz')
+
+        with io.open(filename, 'rb') as fr:
+            fh = VLBIFileReaderBase(fr)
+            fh.seek(2)
+            assert fh.read(2) == b'cd'
+            assert fh.tell() == 4
+            with fh.temporary_offset():
+                assert fh.seek(1) == 1
+                assert fh.read(2) == b'bc'
+                assert fh.tell() == 3
+            assert fh.tell() == 4
+            with fh.temporary_offset(-2, 2):
+                assert fh.read(1) == b'y'
+                assert fh.tell() == 25
+            assert fh.tell() == 4
+
+    def test_pickle(self, tmpdir):
+        filename = str(tmpdir.join('test.dat'))
+        with io.open(filename, 'wb') as fw:
+            fw.write(b'abcdefghijklmnopqrstuvwxyz')
+
+        with VLBIFileReaderBase(io.open(filename, 'rb')) as fh:
+            assert fh.read(2) == b'ab'
+            pickled = pickle.dumps(fh)
+            with pickle.loads(pickled) as fh2:
+                assert fh2.tell() == 2
+                assert fh2.read(2) == b'cd'
+                fh2.seek(-2, 2)
+                assert fh2.read(2) == b'yz'
+
+            # cannot read closed file
+            with pytest.raises(ValueError):
+                fh2.read()
+
+            assert fh.tell() == 2
+            assert fh.read(2) == b'cd'
+
+        with pickle.loads(pickled) as fh3:
+            assert fh3.read(2) == b'cd'
+
+        # cannot read closed file
+        with pytest.raises(ValueError):
+            fh3.read()
+
+        closed = pickle.dumps(fh)
+        with pickle.loads(closed) as fh4:
+            assert fh4.closed
+            with pytest.raises(ValueError):
+                fh4.read()
+
+        with VLBIFileBase(io.open(filename, 'wb')) as fw:
+            with pytest.raises(TypeError):
+                pickle.dumps(fw)
+
 
 class TestSqueezeAndSubset:
     def setup(self):
-        self.other_args = dict(fh_raw=None, header0=None, bps=1,
+        header0 = namedtuple('Header', 'frame_nbytes')(8)
+        self.other_args = dict(fh_raw=None, header0=header0, bps=1,
                                complex_data=False, samples_per_frame=1000,
-                               sample_rate=10000*u.Hz, fill_value=0.,
-                               verify=True)
+                               sample_rate=10000*u.Hz)
         self.sample_shape_maker = namedtuple('SampleShape',
                                              'n0, n1, n2, n3, n4')
         self.unsliced_shape = (1, 21, 33, 1, 2)
@@ -522,16 +584,16 @@ class TestSqueezeAndSubset:
 
         return StreamWriterWithShape(
             unsliced_shape=unsliced_shape or self.unsliced_shape,
-            subset=None, squeeze=squeeze, **self.other_args)
+            squeeze=squeeze, **self.other_args)
 
     def test_sample_shape_and_squeeze(self):
         # Tests stream base's sample and squeezing routines.
         # Try tuple only.
         sb = VLBIStreamBase(unsliced_shape=self.unsliced_shape,
-                            subset=None, squeeze=False, **self.other_args)
+                            squeeze=False, **self.other_args)
         assert sb.sample_shape == self.unsliced_shape
         sb = VLBIStreamBase(unsliced_shape=self.unsliced_shape,
-                            subset=None, squeeze=True, **self.other_args)
+                            squeeze=True, **self.other_args)
         assert sb.sample_shape == self.squeezed_shape
 
         # Try reader with equivalent sample shape.
